
clear all;
close all;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate Gaussian mixture data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rand('seed',0);
N=500;
d = 7; % increase d to append irrelevant features
anom_frac = 0.1;
n_null = ceil(N*(1-anom_frac));
n_null1 = ceil(n_null/2);
n_null2 = n_null - n_null1;
n_anom = N-n_null;
% generate null data
xnull = [mvg(n_null1,2,[0 0]', [1 .7; .7 1]), ...
    mvg(n_null2,2,[0 -2]', [1 -.95; -.95 1])];
% generate some uniform anomalies
if 1 % anomalies supported on box
    minx=min(xnull,[],2);
    maxx=max(xnull,[],2);
    h = 0.1;
    newmin = minx - h*(maxx - minx);
    newmax = maxx + h*(maxx - minx);
    xanom = repmat(newmin,1,n_anom) + rand(2,n_anom).*repmat(newmax-newmin,1,n_anom);
    % assemble mixture data
    x = [xnull xanom];
    if d > 2 % add irrelevant features
        avmin = mean(newmin); avmax = mean(newmax);
        x = [x; repmat(avmin,d-2,N) + rand(d-2,N).*repmat(avmax-avmin,d-2,N)];
    end
else % anomalies supported on disc; used when testing skewed reference measure
    max_norm = max(sqrt(sum(xnull.^2)));
    h = 0.1;
    max_rad = max_norm*(1+h);
    utmp = randn(2,n_anom);
    xanom = max_rad * ( utmp./repmat(sqrt(sum(utmp.^2)),2,1) ) .* repmat((rand(1,n_anom)).^(1/2),2,1);
    x = [xnull xanom];
    if d > 2
        warning('Irrelevant features not available for radially uniform anomalies')
    end
end
anom_idx_unsorted = n_null+1:N;    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Set MN-SCAnn options. Type "help mnscann" for details
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

opts.bandwidth = -1; 
opts.n_grid = 10000; % 10000 used to generate most plots; 
opts.supp_shape = 1;
opts.ref_measure = 1;
opts.show_movie = 0;
opts.show_contour = 0;
opts.show_roc = 0;
opts.show_iv = 1;
opts.show_smooth = 1;
opts.verbose = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Run MN-SCAnn
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

output = mnscann(x,opts);
        
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Visualize results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% organize data
[gamma_sort,gamma_idx] = sort(output.gammas);
beta_sort = sort(output.betas);
extreme_idx = find(output.extreme(gamma_idx));
nonextreme_idx = find(~output.extreme(gamma_idx));
anom_idx_sorted = find(ismember(gamma_idx,anom_idx_unsorted));
anom_frac = length(anom_idx_sorted)/N;

% plot annotation values and indicate anomalies
figure
plot(nonextreme_idx,gamma_sort(nonextreme_idx),'.') % plot gamma curve
hold on
plot(1:N,beta_sort,'r--') % plot beta curve
stem(anom_idx_sorted,gamma_sort(anom_idx_sorted)) 
warning off; % warning issued if extreme points not plotted
legend('gammas','betas','anomalies','Location','NorthWest'); 
warning on
xlabel('Index of ranked points')
ylabel('Annotation')

disp(['Estimated pi = ' num2str(output.pi_est)])
