function [data,anom_idx_unsorted] = load_abilene(week,router,anom_prct)
%
% [x,idx] = load_abilene(week,router)
% 
% week: 1-4
% router: (dimensionality in parens)
%   1 = ATLA (6)
%   2 = CHIN (4)
%   3 = DNVR (6)
%   4 = HSTN (6)
%   5 = IPLS (6)
%   6 = KSCY (8)
%   7 = LOSA (4)
%   8 = NYCM (4)
%   9 = SNVA (8)
%   10 = STTL (4)
%   11 = WASH (4)
% anom_prct: .95, .975, .99, .995, .999
%
% x = data matrix, d x n_train
% idx = vector of the indices of anomalies, as determined by global PCA
% method (Lakhina et al.) with a false alarm rate of anom_prct

switch week
    case 1
        load abilene_data/Y
        week_data = Y;
    case 2      
        load abilene_data/Y2
        week_data = Y2;
    case 3
        load abilene_data/Y3
        week_data = Y3;
    case 4
        load abilene_data/Y4
        week_data = Y4;
end

links = {
    [1 9 2 12 3 29], % ATLA
    [4 13 5 21], % CHIN
    [6 15 7 23 8 27], % DNVR
    [9 1 10 16 11 19], % HSTN
    [12 2 13 4 14 17], % IPLS
    [15 6 16 10 17 14 18 24], % KSCY
    [19 11 20 25], % LOSA
    [21 5 22 30], % NYCM
    [23 7 24 18 25 20 26 28], % SNVA
    [27 8 28 26], % STTL
    [29 3 30 22] % WASH
};

data = week_data(:,links{router})';

% determine outliers
A = dlmread('abilene_data/pca_anom_95_apr03.txt');

switch anom_prct
    case .95
        A = dlmread('abilene_data/pca_anom_95_apr03.txt');
    case .975
        A = dlmread('abilene_data/pca_anom_975_apr03.txt');        
    case .99
        A = dlmread('abilene_data/pca_anom_99_apr03.txt');        
    case .995
        A = dlmread('abilene_data/pca_anom_995_apr03.txt');        
    case.999
        A = dlmread('abilene_data/pca_anom_999_apr03.txt');
end

% identify anomalies and prepare for plotting
anom_idx_unsorted = [];
for p=1:length(links{router})
    for q = 1:size(A,1)
        if A(q,2) == links{router}(p)
             anom_idx_unsorted = [anom_idx_unsorted, A(q,1)];
        end                
    end
end
anom_idx_unsorted = unique(anom_idx_unsorted);


