function [roc_est,dens_est,pi_est]=roc_lin_spline(vols,masses,params)
%
% force beta to be concave function of vols
% 
%

if nargin < 3
    params.show_smooth = 0;
end

[masses,idx]=sort(masses);
N = length(masses);
vols=vols(idx);
Nzero = sum(vols < sqrt(eps)); % min positive vol is (1/params.n_grid)*0.5

scale = 1;

% set knots
K = 10; % number of segments in spline; five vols per segment
knot_idx = unique(ceil(logspace(0,log10(N-Nzero),K))); % adaptive exponential
knot_idx(end) = [];
t = [0 fliplr(1-knot_idx/N) 1];
K = length(t)-1; % might have lost some with 'unique' operation above

% determine which points are in which intervals
[counts,bins]=histc(masses,t);
bins(find(bins==K+1))=K;
counts(K)=counts(K+1)+counts(K);
counts(K+1)=[];

t(1) = []; % remove zero, add back later; makes indexing easier in constucting 
% A, B, so that t(1) is the first positive knot, and zero is the first knot

% we're going to minimize the norm squared of y - A*theta
A = zeros(N,K-1);

A(1:counts(1),1) = masses(1:counts(1))'/t(1); % first block
cumcounts = cumsum(counts);
for k=2:K-1
    A(cumcounts(k-1)+1:cumcounts(k),k-1)=(t(k) - masses(cumcounts(k-1)+1:cumcounts(k))')/(t(k)-t(k-1));
    A(cumcounts(k-1)+1:cumcounts(k),k)=(masses(cumcounts(k-1)+1:cumcounts(k))'-t(k-1))/(t(k)-t(k-1));
end
A(end-counts(K)+1:end,K-1)=(1-masses(end-counts(K)+1:end)')/(1-t(K-1));    % last block

y = vols';
y(end-counts(K)+1:end) = y(end-counts(K)+1:end) - (masses(end-counts(K)+1:end)'-t(K-1))/(1-t(K-1));
y = y*scale;

H = A'*A;
f = -A'*y;

B = zeros(K,K-1);
B(1,1)=-1;
% set second row
B(2,1) = t(2); B(2,2) = -t(1);
% set upper diagonal
for k=3:K-1 
    B(k,k) = t(k-2) - t(k-1);
end
% set middle diagonal
for k=3:K 
    B(k,k-1) = t(k) - t(k-2);
end
% set lower diagonal
for k=3:K 
    B(k,k-2) = t(k-1) - t(k);
end

b = zeros(K,1);
b(end) = t(K-1)-t(K-2);

% solve quadratic program
theta0 = scale*[1/K:1/K:(K-1)/K]'/2; % initial guess
optimset('MaxIter',100000,'TolX',10^(-50),'TolFun',10^(-50));
warning off; % otherwise quadprog issues warning about switching to medium scale method
[theta,fval,exitflag,output] = quadprog(H,f,B,b,[],[],[],[],theta0);
warning on;

theta = [0; theta/scale; 1]; % add endpoints

t = [0, t]; % add the first knot back in
for i=1:N
    k = bins(i);
    roc_est(i) = (theta(k)*(t(k+1) - masses(i)) + theta(k+1)*(masses(i)- t(k)))/(t(k+1)-t(k));
end

%if sum(find(diff([0 roc_est 1])==0)) > 0
%    keyboard
%end

% estimate piecewise constant density
dens_est = diff([0 masses 1])./diff([0 roc_est 1]); 
% may think of roc an an (inverted) empirical CDF
% 1/N = change in mass between consecutive empirical CDF values
% 1/2N = change in mass between last CDF value and endpoint
% this implementation has masses = [1/2N, 3/2N, ..., (2N-1)/2N]


% estimate pi; don't simply take right endpoint because of drop-off effect
%pi_est = dens_est(end);
[dum,pi_est_idx] = max(find(roc_est <= .95));
pi_est = dens_est(pi_est_idx+1); % value of density at .95
% note: by not taking the slope of the last segment, it can happen that
% some gamma values actually exceed 1. a satisfactory fix is to simply
% reassign gammas = min(gammas,1)

if params.show_smooth
    figure
    plot(masses,vols,'.');
    hold on
    plot(masses,roc_est);
    xlabel('masses')
    ylabel('volumes')
    title(['pi est = ' num2str(pi_est)])
    legend('unsmoothed','spline','Location','Northwest')    
    axis([0 1 0 1])
    if 0 % diagnostic plot that shows breakpoints clearly
        for k = 2:K+1
            plot([t(k) t(k)],[0 1],'r') 
        end
    end
    drawnow
end

%gammas = pi_est*(1-roc_est)./(1-masses);
%keyboard

% return to original order
[stuff2,idx2]=sort(idx);
roc_est = roc_est(idx2);
